package edu.travel.commodity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import edu.travel.adapter.service.tenant.TenantAdapter;
import edu.travel.commodity.entity.ShopUserSession;
import edu.travel.commodity.service.ShopMessageService;
import edu.travel.commodity.service.ShopUserSessionService;
import edu.travel.commodity.utils.PageUtil;
import edu.travel.exception.BaseException;
import edu.travel.remote.dto.MessageDto;
import edu.travel.remote.dto.SaveMessageDto;
import edu.travel.remote.feign.mode.vo.tenant.EduTenantVo;
import edu.travel.remote.vo.MessageVo;
import edu.travel.rpc.RPCBaseResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import edu.travel.commodity.mapper.ShopMessageMapper;
import edu.travel.commodity.entity.ShopMessage;

import java.util.*;

@Service
public class ShopMessageServiceImpl extends ServiceImpl<ShopMessageMapper, ShopMessage> implements ShopMessageService {
    @Autowired
    private TenantAdapter tenantAdapter;
    @Autowired
    private ShopUserSessionService shopUserSessionService;


    @Override
    public Page<MessageVo> getMessage(MessageDto messageDto) {
        LambdaQueryWrapper<ShopMessage> query = Wrappers.<ShopMessage>lambdaQuery()
                .eq(ShopMessage::getSessionId, messageDto.getSessionId())
                .orderByDesc(ShopMessage::getCreateTime);
        Page<ShopMessage> page = page(new Page<>(messageDto.getCurrentPage(), messageDto.getPageSize()), query);
        Page<MessageVo> pageEntity = PageUtil.toPageEntity(page, MessageVo.class);
        List<ShopMessage> records = page.getRecords();

        Set<String> userIds = new HashSet<>();
        if(!ObjectUtil.isEmpty(records)){
            ShopMessage message = records.get(0);
            userIds.add(message.getToUserId());
            userIds.add(message.getSendUserId());
        }
        RPCBaseResponse<List<EduTenantVo>> result = tenantAdapter.getTenantByIds(userIds);
        if(result.getCode()!=200){
            throw new BaseException(result.getCode(),result.getMsg());
        }

        List<EduTenantVo> data = result.getData();

        HashMap<String, EduTenantVo> map = new HashMap<>();
        for (EduTenantVo datum : data) {
            map.put(datum.getId(),datum);
        }
        //填充用户
        if(!ObjectUtil.isEmpty(data)){
            List<MessageVo> list = pageEntity.getRecords();
            for (MessageVo messageVo : list) {
                EduTenantVo from = map.get(messageVo.getSendUserId());
                if(!ObjectUtil.isEmpty(from)){
                    messageVo.setSendUserInfo(from);
                }

                EduTenantVo to = map.get(messageVo.getToUserId());
                if(!ObjectUtil.isEmpty(to)){
                    messageVo.setToUserInfo(to);
                }
            }
        }
        return pageEntity;
    }

    @Override
    public Boolean savaMessage(SaveMessageDto params) {
        ShopMessage bean = BeanUtil.toBean(params, ShopMessage.class);
        return save(bean);
    }
}
