package edu.travel.commodity.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import edu.travel.adapter.service.country.CountryAdapter;
import edu.travel.commodity.entity.ShopParameters;
import edu.travel.commodity.entity.ShopProductParameters;
import edu.travel.commodity.mapper.ShopParametersMapper;
import edu.travel.commodity.service.ShopParametersService;
import edu.travel.commodity.service.ShopProductParametersService;
import edu.travel.commodity.utils.PageUtil;
import edu.travel.remote.dto.ParametersDto;
import edu.travel.remote.dto.ProductSpecDto;
import edu.travel.remote.vo.ParametersVo;
import edu.travel.remote.vo.ShopParametersVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.service.SysServiceImpl;
import edu.travel.vo.BaseCountryServeVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class ShopParametersServiceImpl extends SysServiceImpl<ShopParametersMapper, ShopParameters> implements ShopParametersService {
    @Autowired
    private ShopProductParametersService shopProductParametersService;
    @Autowired
    private ShopParametersMapper shopParametersMapper;
    @Autowired
    private CountryAdapter countryAdapter;

    @Override
    public Page<ShopParametersVo> getShopParameters(ProductSpecDto param) {
        LambdaQueryWrapper<ShopProductParameters> query = Wrappers.<ShopProductParameters>lambdaQuery().eq(ShopProductParameters::getProductId, param.getProductId());

        Page<ShopProductParameters> page = shopProductParametersService.page(new Page<>(param.getCurrentPage(), param.getPageSize()), query);
        //分页获取参数值和参数ID
        Page<ShopParametersVo> pageEntity = PageUtil.toPageEntity(page, ShopParametersVo.class);
        List<ShopParametersVo> list = pageEntity.getRecords();
        if(ObjectUtil.isEmpty(list)){
            return pageEntity;
        }
        //参数IDS
        Set<String> collect = list.stream().map(ShopParametersVo::getParametersId).collect(Collectors.toSet());
        //参数值
        List<ShopParameters> parameters = lambdaQuery().in(ShopParameters::getId, collect).list();
        HashMap<String, ShopParameters> map = new HashMap<>();
        for (ShopParameters parameter : parameters) {
            map.put(parameter.getId().toString(), parameter);
        }
        //封装返回结果
        for (ShopParametersVo shopParametersVo : list) {
            ShopParameters shopParameters = map.get(shopParametersVo.getParametersId());
            if(ObjectUtil.isNotEmpty(shopParameters)){
                shopParametersVo.setParametersName(shopParameters.getParametersName());
                shopParametersVo.setIsNecessary(shopParameters.getIsNecessary());
            }
        }
        return pageEntity;
    }

    @Override
    public Page<ParametersVo> getAllParameters(ParametersDto param) {
        LambdaQueryWrapper<ShopParameters> query = Wrappers.<ShopParameters>lambdaQuery().
                eq(ShopParameters::getCountryId, param.getCountryId());
        if(param.getIsNecessary().equals("0")){
            query.eq(ShopParameters::getIsNecessary, param.getIsNecessary());
        }
        Page<ShopParameters> page = this.page(new Page<>(param.getCurrentPage(), param.getPageSize()),query);
        return PageUtil.toPageEntity(page, ParametersVo.class);
    }

    @Override
    public List<ShopParameters> getAllList() {
        return getListLink(null);
    }

    @Override
    public RPCBaseResponse<Page<ShopParameters>> getAllParametersPage(ParametersDto parametersDto) {
        // Step 1: 查询参数数据，确保并计算总记录数
        List<ShopParametersVo> shopParametersVoList = shopParametersMapper.selectParametersWithCountry(parametersDto);
        int total = shopParametersMapper.selectParametersCount(parametersDto);

        // Step 2: 将 ShopParametersVo 转换为 ShopParameters
        List<ShopParameters> shopParametersList = shopParametersVoList.stream()
                .map(this::convertVoToEntity) // 使用转换方法
                .collect(Collectors.toList());

        // Step 3: 提取国家服务ID
        Set<String> countryServeIds = shopParametersList.stream()
                .map(ShopParameters::getCountryId)
                .collect(Collectors.toSet());

        // Step 4: 查询国家信息
        Map<String, ShopParametersVo> countryMap = fetchCountryData(countryServeIds);

        // Step 5: 将国家信息放入参数列表中
        for (ShopParameters shopParameters : shopParametersList) {
            if (shopParameters.getProductParametersMap() == null) {
                shopParameters.setProductParametersMap(new HashMap<>());
            }
            if (countryMap.containsKey(shopParameters.getCountryId())) {
                shopParameters.getProductParametersMap().put("country", countryMap.get(shopParameters.getCountryId()));
            }
        }

        // Step 6: 封装返回结果
        Page<ShopParameters> shopParametersPage = new Page<>();
        shopParametersPage.setRecords(shopParametersList);
        shopParametersPage.setTotal(total);
        shopParametersPage.setCurrent(parametersDto.getCurrentPage());
        shopParametersPage.setSize(parametersDto.getPageSize());

        // Step 7: 返回结果
        return new RPCBaseResponse<>(200, "SUCCESS", shopParametersPage);
    }
    /**
     * 将 ShopParametersVo 转换为 ShopParameters
     */
    private ShopParameters convertVoToEntity(ShopParametersVo parametersVo) {
        ShopParameters parameters = new ShopParameters();
        // 复制各个属性
        parameters.setId(parametersVo.getId());
        parameters.setParametersName(parametersVo.getParametersName());
        parameters.setIsNecessary(parametersVo.getIsNecessary());
        parameters.setProject(parametersVo.getProject());
        parameters.setCreateTime(parametersVo.getCreateTime());
        parameters.setCreateUserId(parametersVo.getCreateUserId());
        parameters.setUpdateTime(parametersVo.getUpdateTime());
        parameters.setUpdateUserId(parametersVo.getUpdateUserId());
        parameters.setDeleteFlag(parametersVo.getDeleteFlag());
        parameters.setCountryId(parametersVo.getCountryId());
        return parameters;
    }

    /**
     * 提取国家数据
     */
    private Map<String, ShopParametersVo> fetchCountryData(Set<String> countryServeIds) {
        Map<String, ShopParametersVo> countryMap = new HashMap<>();
        for (String countryServeId : countryServeIds){
            BaseCountryServeVo country = countryAdapter.getFormId(countryServeId).getData();
            if (country != null){
                ShopParametersVo shopParametersVo = new ShopParametersVo();
                shopParametersVo.setCountryId(countryServeId);
                shopParametersVo.setMap(new HashMap<>());
                shopParametersVo.getMap().put("countryNameZh", country.getCountryNameZh());
                shopParametersVo.getMap().put("countryNameEn", country.getCountryNameEn());
                shopParametersVo.getMap().put("countryNameLocal", country.getCountryNameLocal());
                countryMap.put(countryServeId, shopParametersVo);
            }
        }
        return countryMap;
    }

}
