package edu.travel.commodity.web;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import edu.travel.commodity.entity.ShopOpinionType;
import edu.travel.commodity.service.ShopOpinionTypeService;
import edu.travel.remote.dto.ShopOpinionTypeDto;
import edu.travel.remote.commodity.ShopOpinionTypeRemoteController;
import edu.travel.remote.vo.OpinionTypeVo;
import edu.travel.remote.vo.ShopOpinionTypeVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.web.BaseController;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

/**
* 意见类型(shop_opinion_type)表控制层
*
* @author xxxxx
*/
@RestController
@RequestMapping("/shopOpinionType")
public class ShopOpinionTypeController extends BaseController<ShopOpinionType> implements ShopOpinionTypeRemoteController {
/**
* 服务对象
*/
@Autowired
private ShopOpinionTypeService shopOpinionTypeService;

    /**
     *  根据语言ID查询意见类型
     */
    @GetMapping("/getType")
    public RPCBaseResponse<List<OpinionTypeVo>> getType(@RequestParam("languageId") String languageId) {
        LambdaQueryChainWrapper<ShopOpinionType> list = shopOpinionTypeService.lambdaQuery().eq(ShopOpinionType::getLanguageId, languageId)
                .orderByDesc(ShopOpinionType::getCreateTime);
        return RPCBaseResponse.success(BeanUtil.copyToList(list.list(), OpinionTypeVo.class));
    }

    /**
     * 连表查询意见类型
     */
    @GetMapping("/getAllOpinionTypeLanguage")
    public RPCBaseResponse<IPage<ShopOpinionTypeVo>> getAllOpinionTypeLanguage(ShopOpinionTypeDto dto) {
        return shopOpinionTypeService.getAllOpinionTypeLanguage(dto);
    }

    /**
     * 通过id查询意见类型
     * @param id
     * @return
     */
    @Override
    @GetMapping("/getFormId")
    public RPCBaseResponse<ShopOpinionTypeVo> getFormId(String id) {
        RPCBaseResponse<ShopOpinionType> shopOpinionTypeRPCBaseResponse = super.getId(id);
        RPCBaseResponse<ShopOpinionTypeVo> shopOpinionTypeVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopOpinionTypeRPCBaseResponse, shopOpinionTypeVoRPCBaseResponse);
        return shopOpinionTypeVoRPCBaseResponse;
    }

    /**
     * 更新意见类型
     * @param entity
     * @return
     */
    @Override
    @PostMapping("/updateFormTarget")
    public RPCBaseResponse<ShopOpinionTypeVo> updateTargetFormId(ShopOpinionTypeDto entity) {
        ShopOpinionType shopOpinionType = new ShopOpinionType();
        BeanUtils.copyProperties(entity, shopOpinionType);
        RPCBaseResponse<ShopOpinionType> shopOpinionTypeRPCBaseResponse = super.updateTargetById(shopOpinionType);
        RPCBaseResponse<ShopOpinionTypeVo> shopOpinionTypeVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopOpinionTypeRPCBaseResponse, shopOpinionTypeVoRPCBaseResponse);
        return shopOpinionTypeVoRPCBaseResponse;
    }

    /**
     * 新增意见类型
     * @param entity
     * @return
     */
    @Override
    @PostMapping("/saveFormTarget")
    public RPCBaseResponse<ShopOpinionTypeVo> saveFormTarget(ShopOpinionTypeDto entity) {
        ShopOpinionType shopOpinionType = new ShopOpinionType();
        BeanUtils.copyProperties(entity, shopOpinionType);
        RPCBaseResponse<ShopOpinionType> shopOpinionTypeRPCBaseResponse = super.saveTarget(shopOpinionType);
        RPCBaseResponse<ShopOpinionTypeVo> shopOpinionTypeVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopOpinionTypeRPCBaseResponse, shopOpinionTypeVoRPCBaseResponse);
        return shopOpinionTypeVoRPCBaseResponse;
    }

    /**
     * 删除意见类型
     * @param ids
     * @return
     */
    @Override
    @DeleteMapping("/deleteFormId")
    public RPCBaseResponse<ShopOpinionTypeVo> deleteTargetFormId(List<String> ids) {
        RPCBaseResponse<ShopOpinionType> shopOpinionTypeRPCBaseResponse = super.deleteTargetById(ids);
        RPCBaseResponse<ShopOpinionTypeVo> shopOpinionTypeVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopOpinionTypeRPCBaseResponse, shopOpinionTypeVoRPCBaseResponse);
        return shopOpinionTypeVoRPCBaseResponse;
    }

    /**
     * 获取所有意见类型信息
     * @return
     */
    @Override
    @GetMapping("/getAllForm")
    public RPCBaseResponse<List<ShopOpinionTypeVo>> getAllForm() {
        RPCBaseResponse<List<ShopOpinionType>> countryRPCBaseResponse = super.listAll();
        RPCBaseResponse<List<ShopOpinionTypeVo>> baseCountryVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(countryRPCBaseResponse, baseCountryVoRPCBaseResponse);
        return baseCountryVoRPCBaseResponse;
    }
}
