package edu.travel.commodity.web;

import edu.travel.commodity.entity.ShopProductParameters;
import edu.travel.commodity.service.ShopProductParametersService;
import edu.travel.remote.commodity.ShopProductParametersRemoteController;
import edu.travel.remote.dto.ShopProductParametersDto;
import edu.travel.remote.vo.ShopProductParametersVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.web.BaseController;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 商品参数值(shop_product_parameters)表控制层
 */
@RestController
@RequestMapping("/shop_product_parameters")
public class ShopProductParametersController extends BaseController<ShopProductParameters> implements ShopProductParametersRemoteController {
    /**
     * 服务对象
     */
    @Autowired
    private ShopProductParametersService shopProductParametersService;

    /**
     * 通过id查找商品参数值
     *
     * @param id
     * @return
     */
    @Override
    @GetMapping("/getFormId")
    public RPCBaseResponse<ShopProductParametersVo> getFormId(String id) {
        RPCBaseResponse<ShopProductParameters> shopProductParametersRPCBaseResponse = super.getId(id);
        RPCBaseResponse<ShopProductParametersVo> shopProductParametersVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopProductParametersRPCBaseResponse, shopProductParametersVoRPCBaseResponse);
        return shopProductParametersVoRPCBaseResponse;
    }

    /**
     * 更新商品参数值
     * @param entity
     * @return
     */
    @Override
    @PostMapping("/updateTargetFormId")
    public RPCBaseResponse<ShopProductParametersVo> updateTargetFormId(ShopProductParametersDto entity) {
        ShopProductParameters shopProductParameters = new ShopProductParameters();
        BeanUtils.copyProperties(entity, shopProductParameters);
        RPCBaseResponse<ShopProductParameters> shopProductParametersRPCBaseResponse = super.updateTargetById(shopProductParameters);
        RPCBaseResponse<ShopProductParametersVo> shopProductParametersVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopProductParametersRPCBaseResponse, shopProductParametersVoRPCBaseResponse);
        return shopProductParametersVoRPCBaseResponse;
    }

    /**
     * 新增商品参数值
     * @param entity
     * @return
     */
    @Override
    @PostMapping("/saveFormTarget")
    public RPCBaseResponse<ShopProductParametersVo> saveFormTarget(ShopProductParametersDto entity) {
        ShopProductParameters shopProductParameters = new ShopProductParameters();
        BeanUtils.copyProperties(entity, shopProductParameters);
        RPCBaseResponse<ShopProductParameters> shopProductParametersRPCBaseResponse = super.saveTarget(shopProductParameters);
        RPCBaseResponse<ShopProductParametersVo> shopProductParametersVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopProductParametersRPCBaseResponse, shopProductParametersVoRPCBaseResponse);
        return shopProductParametersVoRPCBaseResponse;
    }

    /**
     * 删除商品参数值
     *
     * @param ids
     * @return
     */
    @Override
    @PostMapping("/deleteTargetFormId")
    public RPCBaseResponse<ShopProductParametersVo> deleteTargetFormId(List<String> ids) {
        RPCBaseResponse<ShopProductParameters> shopProductParametersRPCBaseResponse = super.deleteTargetById(ids);
        RPCBaseResponse<ShopProductParametersVo> shopProductParametersVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopProductParametersRPCBaseResponse, shopProductParametersVoRPCBaseResponse);
        return shopProductParametersVoRPCBaseResponse;
    }

    /**
     * 获取所有商品参数值
     *
     * @return
     */
    @Override
    @GetMapping("/getAllForm")
    public RPCBaseResponse<List<ShopProductParametersVo>> getAllForm() {
        RPCBaseResponse<List<ShopProductParameters>> shopProductParametersRPCBaseResponse = super.listAll();
        RPCBaseResponse<List<ShopProductParametersVo>> shopProductParametersVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopProductParametersRPCBaseResponse, shopProductParametersVoRPCBaseResponse);
        return shopProductParametersVoRPCBaseResponse;
    }
}
