package edu.travel.commodity.web;

import edu.travel.commodity.entity.ShopProductSku;
import edu.travel.commodity.service.ShopProductSkuService;
import edu.travel.remote.commodity.ShopProductSkuRemoteController;
import edu.travel.remote.dto.AddShopOrderDto;
import edu.travel.remote.dto.PayShopOrderDto;
import edu.travel.remote.vo.ShopSkuSpecValueVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.web.BaseController;
import org.springframework.web.bind.annotation.*;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
* 商品SKU表(shop_product_sku)表控制层
*
* @author xxxxx
*/
@RestController
@RequestMapping("/shopProductSku")
public class ShopProductSkuController extends BaseController<ShopProductSku> implements ShopProductSkuRemoteController {
/**
* 服务对象
*/
    @Autowired
    private ShopProductSkuService shopProductSkuService;


    /**
     *  通过sku查询sku规格值
     * @param skuId
     * @return {@link RPCBaseResponse }<{@link Map }<{@link String }, {@link List }<{@link ShopSkuSpecValueVo }>>>
     */

    @GetMapping("/getSkuValuesById")
    public RPCBaseResponse<Map<String, List<ShopSkuSpecValueVo>>> getSkuValuesById(@RequestParam("skuId") Set<String> skuId){
        return RPCBaseResponse.success(shopProductSkuService.getSkuValuesById(skuId));
    }

    /**
     *  商品下单
     * @param params
     * @return {@link RPCBaseResponse }<{@link String }>
     */

    @PostMapping("/shopProductSku")
    public RPCBaseResponse<Void> shopProductSku(@RequestBody AddShopOrderDto params){
        shopProductSkuService.shopProductSku(params);
        return RPCBaseResponse.success();
    }

    /**
     *  支付商品订单
     * @param payShopOrderDto
     * @return {@link RPCBaseResponse }<{@link String }>
     */

    @PostMapping("/payProductSku")
    public RPCBaseResponse<Void> payProductSku(@RequestBody PayShopOrderDto payShopOrderDto){
        shopProductSkuService.payProductSku(payShopOrderDto);
        return RPCBaseResponse.success();
    }

}
