package edu.travel.commodity.web;
import edu.travel.adapter.service.country.CountryAdapter;
import edu.travel.commodity.constant.BaseConstant;
import edu.travel.commodity.entity.ShopUnit;
import edu.travel.remote.commodity.ShopUnitRemoteController;
import edu.travel.remote.dto.ShopUnitDto;
import edu.travel.remote.vo.ShopUnitVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.vo.BaseCountryServeVo;
import edu.travel.vo.BaseCountryVo;
import edu.travel.web.BaseController;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
* 计件单位表(shop_unit)表控制层
*
* @author xxxxx
*/
@RestController
@RequestMapping("/shopUnit")
public class ShopUnitController extends BaseController<ShopUnit> implements ShopUnitRemoteController {
    @Autowired
    private CountryAdapter countryAdapter;
    /**
     * 通过id查找货币单位(连表）
     * @param id
     * @return
     */
    @Override
    @GetMapping("/getFormId")
    public RPCBaseResponse<ShopUnitVo> getFormId(String id) {
        // Step 1: 获取 ShopUnit 数据
        RPCBaseResponse<ShopUnit> shopUnitRPCBaseResponse = super.getId(id);

        // Step 2: 检查响应是否成功
        if (!Objects.equals(shopUnitRPCBaseResponse.getCode(), BaseConstant.SUCCESS_CODE)) {
            return RPCBaseResponse.error(shopUnitRPCBaseResponse.getMsg());
        }

        // 获取 ShopUnit 对象
        ShopUnit shopUnit = shopUnitRPCBaseResponse.getData();

        // Step 3: 提取 countryId
        Long countryId = shopUnit.getCountryId();

        // Step 4: 调用 countryAdapter 获取国家信息
        RPCBaseResponse<BaseCountryServeVo> countryInfoResponse = countryAdapter.getFormId(countryId.toString());

        // Step 5: 检查国家信息的响应
        if (!Objects.equals(countryInfoResponse.getCode(), BaseConstant.SUCCESS_CODE)) {
            return RPCBaseResponse.error(countryInfoResponse.getMsg());
        }

        // 获取国家信息
        BaseCountryServeVo countryInfo = countryInfoResponse.getData();

        // Step 6: 创建 ShopUnitVo 对象
        ShopUnitVo shopUnitVo = new ShopUnitVo();
        BeanUtils.copyProperties(shopUnit, shopUnitVo);

        // Step 7: 将国家信息放入 map 中
        Map<String, Object> map = new HashMap<>();
        map.put("countryInfo", countryInfo);
        shopUnitVo.setMap(map);

        // Step 8: 返回成功的响应
        return RPCBaseResponse.success(shopUnitVo);
    }


    /**
     * 更新货币单位
     * @param entity
     * @return
     */
    @Override
    @PostMapping("/updateTargetFormId")
    public RPCBaseResponse<ShopUnitVo> updateTargetFormId(@RequestBody ShopUnitDto entity) {
        ShopUnit shopUnit = new ShopUnit();
        BeanUtils.copyProperties(entity, shopUnit);
        RPCBaseResponse<ShopUnit> shopUnitRPCBaseResponse = super.updateTargetById(shopUnit);
        RPCBaseResponse<ShopUnitVo> shopUnitVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopUnitRPCBaseResponse, shopUnitVoRPCBaseResponse);
        return shopUnitVoRPCBaseResponse;
    }

    /**
     * 新增货币单位
     * @param entity
     * @return
     */
    @Override
    @PostMapping("/saveFormTarget")
    public RPCBaseResponse<ShopUnitVo> saveFormTarget(@RequestBody ShopUnitDto entity) {
        ShopUnit shopUnit = new ShopUnit();
        BeanUtils.copyProperties(entity, shopUnit);
        RPCBaseResponse<ShopUnit> shopUnitRPCBaseResponse = super.saveTarget(shopUnit);
        RPCBaseResponse<ShopUnitVo> shopUnitVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopUnitRPCBaseResponse, shopUnitVoRPCBaseResponse);
        return shopUnitVoRPCBaseResponse;
    }

    /**
     * 删除货币单位
     * @param ids
     * @return
     */
    @Override
    @PostMapping("/deleteFormTarget")
    public RPCBaseResponse<ShopUnitVo> deleteTargetFormId(@RequestBody List<String> ids) {
        RPCBaseResponse<ShopUnit> shopUnitRPCBaseResponse = super.deleteTargetById(ids);
        RPCBaseResponse<ShopUnitVo> shopUnitVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopUnitRPCBaseResponse, shopUnitVoRPCBaseResponse);
        return shopUnitVoRPCBaseResponse;
    }

    /**
     * 获取所有货币单位
     * @return
     */
    @Override
    @GetMapping("/getAllForm")
    public RPCBaseResponse<List<ShopUnitVo>> getAllForm() {
        RPCBaseResponse<List<ShopUnit>> shopUnitRPCBaseResponse = super.listAll();
        RPCBaseResponse<List<ShopUnitVo>> shopUnitVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(shopUnitRPCBaseResponse, shopUnitVoRPCBaseResponse);
        return shopUnitVoRPCBaseResponse;
    }
}
