package edu.travel.country.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import edu.travel.country.entity.BaseCountryServe;
import edu.travel.country.mapper.BaseCountryServeMapper;
import edu.travel.dto.BaseCountryDto;
import edu.travel.dto.BaseCountryServeDto;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.service.SysServiceImpl;
import edu.travel.vo.BaseCountryServeVo;
import edu.travel.vo.BaseCountryVo;
import org.springframework.beans.BeanUtils;
import edu.travel.country.entity.BaseCountry;
import edu.travel.country.mapper.BaseCountryMapper;
import edu.travel.country.service.BaseCountryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestBody;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class BaseCountryServiceImpl extends SysServiceImpl<BaseCountryMapper, BaseCountry> implements BaseCountryService {
@Autowired
    private BaseCountryServeMapper baseCountryServeMapper;
    @Override
    public RPCBaseResponse<IPage<BaseCountryVo>> getCountryPage(BaseCountryDto baseCountryDto) {
        Page<BaseCountry> baseCountryPage = new Page<>(baseCountryDto.getCurrentPage(), baseCountryDto.getPageSize());
        IPage<BaseCountry> pageLink = super.getPageLink(new LambdaQueryWrapper<BaseCountry>(), baseCountryPage);
        IPage<BaseCountryVo> pageVoLink = new Page<>();
        BeanUtils.copyProperties(pageLink,pageVoLink);
        RPCBaseResponse<IPage<BaseCountryVo>> pageRPCPageResponse = new RPCBaseResponse<>(200,"SUCCESS",pageVoLink);
        return pageRPCPageResponse;
    }

    //国家树
    @Override
    public RPCBaseResponse<List<BaseCountryVo>> getCountryTree() {
        //查询所有国家数据
        LambdaQueryWrapper<BaseCountry> queryWrapper =  new LambdaQueryWrapper<>();
        //获取所有国际家数据
        List<BaseCountry> baseCountryList = super.getListLink(queryWrapper);

        //构建树形结构
        List<BaseCountryVo>  BaseCountryTree = buildCountryTree(baseCountryList);

        //返回结果
        return new RPCBaseResponse<>(200,"SUCCESS",BaseCountryTree);
    }

    @Override
    public RPCBaseResponse<BaseCountryServeVo> saveFormTargetNew(BaseCountryServeDto entity, HttpServletRequest request) {
        // 创建 BaseCountryServe 对象
        BaseCountryServe baseCountryServe = new BaseCountryServe();

        // 使用 BeanUtils 复制属性
        BeanUtils.copyProperties(entity, baseCountryServe);

        // 从请求头获取 Project
        String project = request.getHeader("Project");
        baseCountryServe.setProject(project); // 设置 Project

        // 获取当前登录用户并设置 create_user_id
        Long currentUserId = getCurrentUserId();
        baseCountryServe.setCreateUserId(String.valueOf(currentUserId));

        baseCountryServeMapper.insert(baseCountryServe);
        // 创建返回的 RPCBaseResponse
        RPCBaseResponse<BaseCountryServeVo> baseCountryServeVoRPCBaseResponse = new RPCBaseResponse<>();
        BaseCountryServeVo baseCountryServeVo = new BaseCountryServeVo();
        BeanUtils.copyProperties(baseCountryServe, baseCountryServeVo); // 复制属性到 VO
        baseCountryServeVoRPCBaseResponse.setData(baseCountryServeVo); // 设置返回数据

        return baseCountryServeVoRPCBaseResponse;
    }

    // 获取当前用户 ID 的方法
    private Long getCurrentUserId() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        Object principal = authentication.getPrincipal();
        Long userId = null;

        if (!(principal instanceof String && "anonymousUser".equals(principal))) {
            String jsonString = JSON.toJSONString(principal);
            JSONObject jsonObject = JSON.parseObject(jsonString);
            userId = jsonObject.getLong("id");
        }

        return userId;
    }

    //树构建方法
    private List<BaseCountryVo> buildCountryTree(List<BaseCountry> baseCountryList) {
        //map，快速查找国家VO对象
        Map<Long,BaseCountryVo> countryMap = new HashMap<>();
        List<BaseCountryVo> rootCountries = new ArrayList<>();

        //构建每个国家VO对象
        for(BaseCountry baseCountry : baseCountryList){
            BaseCountryVo baseCountryVo = new BaseCountryVo();
            baseCountryVo.setId(baseCountry.getId());
            baseCountryVo.setParentId(baseCountry.getParentId());
            baseCountryVo.setImageUrl(baseCountry.getImageUrl());
            baseCountryVo.setCountryNameZh(baseCountry.getCountryNameZh());
            baseCountryVo.setCountryNameEn(baseCountry.getCountryNameEn());
            baseCountryVo.setCountryNameLocal(baseCountry.getCountryNameLocal());
            baseCountryVo.setAreaCode(String.valueOf(baseCountry.getAreaCode()));
            countryMap.put(Long.valueOf(baseCountry.getId()),baseCountryVo);
        }

        //构建树形结构
        for (BaseCountry baseCountry : baseCountryList){
            BaseCountryVo baseCountryVo = countryMap.get(baseCountry.getId());
            Long parentId = Long.valueOf(baseCountry.getParentId());

            if (parentId == 0|| parentId == null){
                rootCountries.add(baseCountryVo);
            }else{
                BaseCountryVo parentCountryVo = countryMap.get(parentId);
                if (parentCountryVo != null){
                    parentCountryVo.getChildren().add(baseCountryVo);
                }
            }
        }
        return rootCountries;
    }

}
