package edu.travel.country.web;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import edu.travel.country.entity.BaseCountry;
import edu.travel.country.service.BaseCountryService;
import edu.travel.dto.BaseCountryDto;
import edu.travel.remote.BaseCountryRemoteController;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.vo.BaseCountryNameAndCodeVo;
import edu.travel.vo.BaseCountryVo;
import edu.travel.web.BaseController;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 国家表(base_country)表控制层
 *
 * @author xxxxx
 */
@RestController
@RequestMapping("/baseCountry")
public class BaseCountryController extends BaseController<BaseCountry> implements BaseCountryRemoteController {

    /**
     * 服务对象
     */
    @Autowired
    private BaseCountryService baseCountryService;

    /**
     * 生成国家树
     *
     * @return
     */
    //国家树
    @GetMapping("/getCountryTree")
    public RPCBaseResponse<List<BaseCountryVo>> getCountryTree() {
        return baseCountryService.getCountryTree();
    }

    /**
     * 国家分页
     */
    @GetMapping("/getCountryPage")
    public RPCBaseResponse<IPage<BaseCountryVo>> getCountryPage(BaseCountryDto baseCountryDTO) {
        return baseCountryService.getCountryPage(baseCountryDTO);
    }

    /**
     * 通过id查询国家
     */
    @Override
    @GetMapping("/getFormId")
    public RPCBaseResponse<BaseCountryVo> getFormId(String id) {
        RPCBaseResponse<BaseCountry> countryRPCBaseResponse = super.getId(id);
        RPCBaseResponse<BaseCountryVo> baseCountryVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(countryRPCBaseResponse, baseCountryVoRPCBaseResponse);
        return baseCountryVoRPCBaseResponse;
    }

    /**
     * 更新国家信息
     */
    @Override
    @PostMapping("/updateTargetFormId")
    public RPCBaseResponse<BaseCountryVo> updateTargetFormId(@RequestBody BaseCountryDto entity) {
        BaseCountry baseCountry = new BaseCountry();
        BeanUtils.copyProperties(entity, baseCountry);
        RPCBaseResponse<BaseCountry> baseCountryRPCBaseResponse = super.updateTargetById(baseCountry);
        RPCBaseResponse<BaseCountryVo> baseCountryVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(baseCountryRPCBaseResponse, baseCountryVoRPCBaseResponse);
        return baseCountryVoRPCBaseResponse;
    }

    /**
     * 新增国家信息
     */
    @Override
    @PostMapping("/saveFormTarget")
    public RPCBaseResponse<BaseCountryVo> saveFormTarget(@RequestBody BaseCountryDto entity) {
        BaseCountry baseCountry = new BaseCountry();
        BeanUtils.copyProperties(entity, baseCountry);
        RPCBaseResponse<BaseCountry> baseCountryRPCBaseResponse = super.saveTarget(baseCountry);
        RPCBaseResponse<BaseCountryVo> baseCountryVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(baseCountryRPCBaseResponse, baseCountryVoRPCBaseResponse);
        return baseCountryVoRPCBaseResponse;
    }

    /**
     * 删除国家
     */
    @Override
    @PostMapping("/deleteTargetFormId")
    public RPCBaseResponse<BaseCountryVo> deleteTargetFormId(@RequestBody List<String> ids) {
        RPCBaseResponse<BaseCountry> baseCountryRPCBaseResponse = super.deleteTargetById(ids);
        RPCBaseResponse<BaseCountryVo> baseCountryVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(baseCountryRPCBaseResponse, baseCountryVoRPCBaseResponse);
        return baseCountryVoRPCBaseResponse;
    }

    /**
     * 获取国家表
     */
    @Override
    @GetMapping("/getAllForm")
    public RPCBaseResponse<List<BaseCountryVo>> getAllForm() {
        RPCBaseResponse<List<BaseCountry>> listRPCBaseResponse = super.listAll();
        RPCBaseResponse<List<BaseCountryVo>> baseCountryVoRPCBaseResponse = new RPCBaseResponse<>();
        BeanUtils.copyProperties(listRPCBaseResponse, baseCountryVoRPCBaseResponse);
        return baseCountryVoRPCBaseResponse;
    }

    /**
     * 获取国家名称和区号
     */
    @GetMapping("/getCountryCode")
    @PreAuthorize("permitAll()")
    public RPCBaseResponse<List<BaseCountryNameAndCodeVo>> getCountryNameAndAreaCode() {
        List<BaseCountry> list = baseCountryService.list(new LambdaQueryWrapper<BaseCountry>()
                .select(BaseCountry::getId,BaseCountry::getCountryNameZh, BaseCountry::getAreaCode, BaseCountry::getCountryNameEn)
                .eq(BaseCountry::getDeleteFlag, 0)
                .ne(BaseCountry::getParentId, 0));
        List<BaseCountryNameAndCodeVo> baseCountryVos = BeanUtil.copyToList(list, BaseCountryNameAndCodeVo.class);
        return new RPCBaseResponse<>(200, "sucess", baseCountryVos);
    }
}
