package edu.travel.tenant.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import edu.travel.remote.feign.mode.dto.tenant.AddOrUpdateRoleDto;
import edu.travel.remote.feign.mode.dto.tenant.AssignRolesOrMenusDto;
import edu.travel.remote.feign.mode.vo.tenant.MenuTreeVo;
import edu.travel.tenant.entity.SysMenu;
import edu.travel.tenant.service.ISysMenuService;
import edu.travel.tenant.service.ISysRoleMenuService;
import edu.travel.tenant.entity.EduTenant;
import edu.travel.tenant.entity.SysRole;
import edu.travel.tenant.service.ISysRoleService;
import edu.travel.service.SysServiceImpl;
import edu.travel.tenant.mapper.SysRoleMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import edu.travel.tenant.entity.SysRoleMenu;

import java.util.*;
import java.util.stream.Collectors;

/**
 * ISysRoleServiceImpl 类。
 * <p>
 * 描述：
 *
 * @author huangwenwen
 * @date 2025/2/6
 */

@Service
public class ISysRoleServiceImpl extends SysServiceImpl<SysRoleMapper, SysRole> implements ISysRoleService {

    @Autowired
    private ISysRoleMenuService iSysRoleMenuService;
    @Autowired
    private ISysMenuService sysMenuService;
    @Autowired
    private ISysRoleMenuService sysRoleMenuService;

    @Override
    @Transactional
    public SysRole addRoleAndAssignMenus(AddOrUpdateRoleDto addOrUpdateRoleDto, EduTenant principal) {

        //参数验证
        Assert.isFalse(addOrUpdateRoleDto==null||addOrUpdateRoleDto.getName()==null,"参数不能为空");

        AssignRolesOrMenusDto assignRolesOrMenusDto = new AssignRolesOrMenusDto();

        //角色
        Assert.isFalse(Objects.equals(querySuperRoleId(), addOrUpdateRoleDto.getId()),"不能操作超级管理员角色");
        SysRole role = this.getOne(new LambdaQueryWrapper<SysRole>().eq(SysRole::getName, addOrUpdateRoleDto.getName()));
        Assert.isFalse(role!=null && !Objects.equals(addOrUpdateRoleDto.getId(), role.getId()), "该角色已存在");
        assignRolesOrMenusDto.setId(addOrUpdateRoleDto.getId());

        //菜单
        List<Long> list = validateMenus(addOrUpdateRoleDto);
        assignRolesOrMenusDto.setIds(list);

        //保存角色并分配菜单
        SysRole sysRole = BeanUtil.copyProperties(addOrUpdateRoleDto, SysRole.class);
        Assert.isFalse(!this.save(sysRole), "角色添加失败");
        assignRolesOrMenusDto.setId(sysRole.getId());
        assign(assignRolesOrMenusDto, principal);

        return sysRole;

    }

    @Override
    @Transactional
    public void deleteByIds(Set<Long> ids) {
        Assert.isFalse(ids.contains(querySuperRoleId()),"不能操作超级管理员角色");
        List<SysRole> listLink = super.list(new LambdaQueryWrapper<SysRole>().in(SysRole::getId, ids));
        Assert.isFalse(listLink == null || listLink.size() != ids.size(), "包含不存在的角色");
        listLink.forEach(sysRole -> {
            Assert.isFalse(sysRole.getDeleteFlag() == 1, "包含已删除角色");
        });
        this.removeByIds(ids);
    }

    @Override
    @Transactional
    public void updateRoleAndMenusById(AddOrUpdateRoleDto addOrUpdateRoleDto) {

        //参数验证
        Assert.isFalse(addOrUpdateRoleDto==null||addOrUpdateRoleDto.getId()==null || addOrUpdateRoleDto.getName()==null,"参数不能为空");

        AssignRolesOrMenusDto assignRolesOrMenusDto = new AssignRolesOrMenusDto();

        //角色
        Assert.isTrue(ObjectUtil.isNotEmpty(this.getById(addOrUpdateRoleDto.getId())),"该角色不存在");
        Assert.isFalse(querySuperRoleId().equals(addOrUpdateRoleDto.getId()),"不能操作超级管理员角色");
        SysRole role = this.getOne(new LambdaQueryWrapper<SysRole>().eq(SysRole::getName, addOrUpdateRoleDto.getName()));
        Assert.isFalse(role!=null && !Objects.equals(addOrUpdateRoleDto.getId(), role.getId()), "该角色已存在");
        assignRolesOrMenusDto.setId(addOrUpdateRoleDto.getId());

        //菜单
        List<Long> list = validateMenus(addOrUpdateRoleDto);
        assignRolesOrMenusDto.setIds(list);

        //保存角色并分配菜单
        Assert.isFalse(!this.saveOrUpdate(BeanUtil.copyProperties(addOrUpdateRoleDto,SysRole.class)), "角色添加失败");
        EduTenant principal = (EduTenant) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        deleteRoleMenu(addOrUpdateRoleDto.getId(),principal);
        assign(assignRolesOrMenusDto, principal);
    }



    @Override
    @Transactional
    public void updateMenusToRole(AssignRolesOrMenusDto assignRolesOrMenusDto, EduTenant principal) {
        assignRolesToUserValidate(assignRolesOrMenusDto);
        deleteRoleMenu(assignRolesOrMenusDto.getId(),principal);
        assign(assignRolesOrMenusDto,principal);
    }

    @Override
    public List<MenuTreeVo> getMenuTreeByRoleId(Long roleId) {
        Assert.isFalse(roleId==null,"参数不能为空");
        Assert.isTrue(ObjectUtil.isNotEmpty(this.getById(roleId)),"该角色不存在");
        List<SysRoleMenu> sysRoleMenus = sysRoleMenuService.list(new LambdaQueryWrapper<SysRoleMenu>().eq(SysRoleMenu::getRoleId, roleId));
            if (ObjectUtil.isEmpty(sysRoleMenus)){
            return new ArrayList<>();
        }
        List<Long> longs = sysRoleMenus.stream().map(sysRoleMenu -> sysRoleMenu.getMenuId()).collect(Collectors.toList());
        List<SysMenu> list = sysMenuService.list(new LambdaQueryWrapper<SysMenu>().in(SysMenu::getId, longs));
        List<MenuTreeVo> treeVoList = sysMenuService.getMenuTreeListByIds(list);
        return treeVoList;
    }

    @Override
    public void enableRoleByIds(Set<Long> list) {
        Assert.isTrue(ObjectUtil.isNotEmpty(list),"请选择角色");
        Long superRoleId = querySuperRoleId();
        list.forEach(id->{
            Assert.isFalse(superRoleId.equals(id),"不能操作超级管理员角色");
        });
        List<SysRole> roleList = this.list(new LambdaQueryWrapper<SysRole>().in(SysRole::getId, list));
        Assert.isFalse(roleList == null || roleList.size() != list.size(), "包含不存在的角色");
        roleList.forEach(sysRole -> {
            Assert.isFalse(sysRole.getDeleteFlag() == 1, "包含已删除角色");
            Assert.isFalse(sysRole.getStatus() == 1, "包含已经启用的角色");
        });
        this.update(new LambdaUpdateWrapper<SysRole>().set(SysRole::getStatus,1).in(SysRole::getId,list));

    }

    @Override
    public void disableRoleByIds(Set<Long> list) {
        Assert.isTrue(ObjectUtil.isNotEmpty(list),"请选择角色");
        Long superRoleId = querySuperRoleId();
        list.forEach(id->{
            Assert.isFalse(superRoleId.equals(id),"不能操作超级管理员角色");
        });
        List<SysRole> roleList = this.list(new LambdaQueryWrapper<SysRole>().in(SysRole::getId, list));
        Assert.isFalse(roleList == null || roleList.size() != list.size(), "包含不存在的角色");
        roleList.forEach(sysRole -> {
            Assert.isFalse(sysRole.getDeleteFlag() == 1, "包含已删除角色");
            Assert.isFalse(sysRole.getStatus() == 0, "包含已经禁用的角色");
        });
        this.update(new LambdaUpdateWrapper<SysRole>().set(SysRole::getStatus,0).in(SysRole::getId,list));
    }

    /**
     * 添加或修改角色参数验证 菜单验证
     */
    public List<Long> validateMenus(AddOrUpdateRoleDto addOrUpdateRoleDto){
        List<Long> menuIds = addOrUpdateRoleDto.getMenuIds();
        if (ObjectUtil.isEmpty(menuIds)) return new ArrayList<>();
        Set<Long> menuIdsSet = new HashSet<>(menuIds);
        List<SysMenu> roleList = sysMenuService.list(new LambdaQueryWrapper<SysMenu>().eq(SysMenu::getDeleteFlag, 0).in(SysMenu::getId, menuIdsSet));
        Assert.isFalse(roleList.isEmpty()||menuIdsSet.size() != roleList.size(), "包含不存在的菜单或重复菜单");
        return menuIds;
    }

    /**
     * 分配角色参数验证
     */
    private void assignRolesToUserValidate(AssignRolesOrMenusDto dto) {
        Assert.isTrue(ObjectUtil.isNotEmpty(dto),"参数不能为空");
        Assert.isTrue(ObjectUtil.isNotEmpty(dto.getId()),"请选择需分配菜单的角色");
        Assert.isTrue(ObjectUtil.isNotEmpty(dto.getIds()),"请选择菜单");
        HashSet<Long> longs = new HashSet<>(dto.getIds());
        Assert.isTrue(longs.size()== dto.getIds().size(),"包含重复的菜单");
        //查询角色
        SysRole sysRole = this.getById(dto.getId());
        Assert.isTrue(ObjectUtil.isNotEmpty(sysRole),"所选角色不存在");
        Assert.isFalse(querySuperRoleId().equals(dto.getId()),"不能操作超级管理员角色");
        //查询菜单列表
        List<SysRole> sysMenuList = super.listByIds(dto.getIds());
        Assert.isTrue(sysMenuList==null||sysMenuList.size()!= dto.getIds().size(),"包含重复的菜单或菜单不存在");
    }

    /**
     * 分配菜单
     */
    private void assign (AssignRolesOrMenusDto dto, EduTenant principal) {
        iSysRoleMenuService.saveOrUpdateBatch(dto.getIds().stream().map(menuId -> {
            SysRoleMenu sysRoleMenu = new SysRoleMenu();
            sysRoleMenu.setRoleId(dto.getId());
            sysRoleMenu.setMenuId(menuId);
            sysRoleMenu.setProject(principal.getProject());
            return sysRoleMenu;
        }).collect(Collectors.toList()));
    }

    /**
     * 删除该角色已有的菜单
     */
    private void deleteRoleMenu(Long roleId,EduTenant principal) {
        iSysRoleMenuService.remove(new LambdaQueryWrapper<SysRoleMenu>()
                .eq(SysRoleMenu::getRoleId, roleId));
    }

    /**
     * 超级管理员的id
     */
    public Long querySuperRoleId(){
        return this.getOne(new LambdaQueryWrapper<SysRole>().eq(SysRole::getName,"超级管理员")).getId();
    }
}
