package edu.travel.ws.service.impl;


import edu.travel.adapter.service.commodity.ShopMessageAdapter;
import edu.travel.adapter.service.commodity.ShopUserSessionAdapter;
import edu.travel.exception.BaseException;
import edu.travel.remote.dto.MessageDto;
import edu.travel.remote.dto.SaveMessageDto;
import edu.travel.remote.vo.ShopUserSessionVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.ws.enums.MessageEnum;
import edu.travel.ws.service.MessageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

@Component
public class ShopMessageService implements MessageService {
    @Autowired
    private ShopUserSessionAdapter shopUserSessionAdapter;
    @Autowired
    private ShopMessageAdapter shopMessageAdapter;

    @Override
    public MessageEnum getCode() {
        return MessageEnum.SHOP_MESSAGE;
    }


    @Override
    public List<String> sendMessage(MessageDto messageDto) {
        //获取接收人
        RPCBaseResponse<ShopUserSessionVo> sessionInfo =
                shopUserSessionAdapter.getSessionInfo(messageDto.getSessionId());
        ShopUserSessionVo data = sessionInfo.getData();
        //判断聊天的人和数据库会话的人 是否是相同
        HashSet<String> ids = new HashSet<>();
        ids.add(data.getUserId());
        ids.add(data.getCustomerId());
        ids.add(messageDto.getSendUserId());
        ids.add(messageDto.getToUserId());
        if(ids.size()!=2){
            throw new BaseException("聊天对象错误");
        }
        //保存消息
        SaveMessageDto saveMessageDto = new SaveMessageDto();

        saveMessageDto.setMessage(messageDto.getContent());
        saveMessageDto.setMessageType(messageDto.getType());
        saveMessageDto.setSendUserId(messageDto.getSendUserId());
        saveMessageDto.setToUserId(messageDto.getToUserId());
        saveMessageDto.setSessionId(data.getId());

        shopMessageAdapter.savaMessage(saveMessageDto);
        ArrayList<String> list = new ArrayList<>();
        list.add(messageDto.getToUserId());
        return list;
    }


}
